@@ -15,7 +15,7 @@
 # along with this library; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307  USA.
 #
-# $Header: /cvsroot/gtk2-perl/gtk2-perl-xs/Glib/Glib.pm,v 1.91 2006/01/01 20:34:33 kaffeetisch Exp $
+# $Header: /cvsroot/gtk2-perl/gtk2-perl-xs/Glib/Glib.pm,v 1.95 2006/01/16 21:44:40 kaffeetisch Exp $
 #
 
 package Glib;
@@ -56,12 +56,14 @@ our %EXPORT_TAGS = (
 			filename_from_unicode
 			filename_to_uri
 			filename_from_uri
+			filename_display_name
+			filename_display_basename
 			/],
 );
 our @EXPORT_OK = map { @$_ } values %EXPORT_TAGS;
 $EXPORT_TAGS{all} = \@EXPORT_OK;
 
-our $VERSION = '1.113';
+our $VERSION = '1.114';
 
 sub dl_load_flags { $^O eq 'darwin' ? 0x00 : 0x01 }
 
@@ -172,6 +174,43 @@ sub tie_properties
 	}
 }
 
+package Glib::Object::_LazyLoader;
+
+use strict;
+no strict qw(refs);
+use vars qw($AUTOLOAD);
+push @Carp::CARP_NOT, __PACKAGE__;
+
+# These two overrides won't keep explicit calls to UNIVERSAL::(isa|can)
+# from breaking if called before anything is loaded, but those should
+# be quite rare.
+
+sub isa {
+	# we really shouldn't get in here at all if $_[0] is undefined.
+	_load (ref($_[0]) || $_[0]);
+	$_[0]->SUPER::isa ($_[1]);
+}
+
+sub can {
+	# we really shouldn't get in here at all if $_[0] is undefined.
+	_load (ref($_[0]) || $_[0]);
+	$_[0]->SUPER::can ($_[1]);
+}
+
+sub AUTOLOAD {
+	(my $method = $AUTOLOAD) =~ s/^.*:://;
+	(my $lazy_class = $AUTOLOAD) =~ s/::[^:]*$//;
+	my $object_or_type = shift;
+
+	_load ($lazy_class);
+
+	die "Something is very broken, couldn't lazy load $lazy_class"
+		if $object_or_type->isa (__PACKAGE__);
+
+	# try again
+	return $object_or_type->$method (@_);
+}
+
 package Glib;
 
 1;
@@ -361,6 +400,9 @@ Example:
 
    $gtkfilesel->set_filename (filename_to_unicode $ARGV[1]);
 
+This function is croak() if the conversion cannot be made, e.g., because the
+utf-8 is invalid.
+
 =item $filename_in_local_encoding = filename_from_unicode $filename
 
 =item $filename_in_local_encoding = Glib->filename_from_unicode ($filename)
@@ -379,6 +421,25 @@ be useful to know that perl currently has no policy at all regarding
 filename issues, if your scalar happens to be in utf-8 internally it will
 use utf-8, if it happens to be stored as bytes, it will use it as-is.
 
+When dealing with filenames that you need to display, there is a much easier
+way, as of Glib 1.120 and glib 2.6.0:
+
+=over 4
+
+=item $uft8_string = filename_display_name ($filename)
+
+=item $uft8_string = filename_display_basename ($filename)
+
+Given a I<$filename> in filename encoding, return the filename, or just
+the file's basename, in utf-8.  Unlike the other functions described above,
+this one is guaranteed to return valid utf-8, but the conversion is not
+necessarily reversible.  These functions are intended to be used for failsafe
+display of filenames, for example in gtk+ labels.
+
+Since gtk+ 2.6, Glib 1.12
+
+=back
+
 
 =head1 EXCEPTIONS
 
@@ -494,6 +555,8 @@ you can also get all of them with the export tag "all".
   filename_to_unicode
   filename_from_uri
   filename_to_uri
+  filename_display_basename
+  filename_display_name
 
 =back
 
@@ -548,7 +611,7 @@ patches and tests here and there.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2003-2005 by muppet and the gtk2-perl team
+Copyright 2003-2006 by muppet and the gtk2-perl team
 
 This library is free software; you can redistribute it and/or modify
 it under the terms of the Lesser General Public License (LGPL).  For 
